@@ -1,8 +1,8 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
-   | Copyright (c) 1997-2015 The PHP Group                                |
+   | Copyright (c) 1997-2016 The PHP Group                                |
    +----------------------------------------------------------------------+
    | This source file is subject to version 3.01 of the PHP license,      |
    | that is bundled with this package in the file LICENSE, and is        |
@@ -30,6 +30,7 @@
 #include "base64.h"
 #include "zend_interfaces.h"
 #include "info.h"
+#include "php_random.h"
 
 #if PHP_WIN32
 #include "win32/winutil.h"
@@ -57,7 +58,7 @@ static char* php_password_get_algo_name(const php_password_algo algo)
 	}
 }
 
-static php_password_algo php_password_determine_algo(const char *hash, const size_t len) 
+static php_password_algo php_password_determine_algo(const char *hash, const size_t len)
 {
 	if (len > 3 && hash[0] == '$' && hash[1] == '2' && hash[2] == 'y' && len == 60) {
 		return PHP_PASSWORD_BCRYPT;
@@ -82,41 +83,39 @@ static int php_password_salt_is_alphabet(const char *str, const size_t len) /* {
 static int php_password_salt_to64(const char *str, const size_t str_len, const size_t out_len, char *ret) /* {{{ */
 {
 	size_t pos = 0;
-	size_t ret_len = 0;
-	unsigned char *buffer;
+	zend_string *buffer;
 	if ((int) str_len < 0) {
 		return FAILURE;
 	}
-	buffer = php_base64_encode((unsigned char*) str, (int) str_len, (int*) &ret_len);
-	if (ret_len < out_len) {
+	buffer = php_base64_encode((unsigned char*) str, str_len);
+	if (ZSTR_LEN(buffer) < out_len) {
 		/* Too short of an encoded string generated */
-		efree(buffer);
+		zend_string_release(buffer);
 		return FAILURE;
 	}
 	for (pos = 0; pos < out_len; pos++) {
-		if (buffer[pos] == '+') {
+		if (ZSTR_VAL(buffer)[pos] == '+') {
 			ret[pos] = '.';
-		} else if (buffer[pos] == '=') {
-			efree(buffer);
+		} else if (ZSTR_VAL(buffer)[pos] == '=') {
+			zend_string_free(buffer);
 			return FAILURE;
 		} else {
-			ret[pos] = buffer[pos];
+			ret[pos] = ZSTR_VAL(buffer)[pos];
 		}
 	}
-	efree(buffer);
+	zend_string_free(buffer);
 	return SUCCESS;
 }
 /* }}} */
 
-static int php_password_make_salt(size_t length, char *ret TSRMLS_DC) /* {{{ */
+static int php_password_make_salt(size_t length, char *ret) /* {{{ */
 {
-	int buffer_valid = 0;
-	size_t i, raw_length;
+	size_t raw_length;
 	char *buffer;
 	char *result;
 
 	if (length > (INT_MAX / 3)) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Length is too large to safely generate");
+		php_error_docref(NULL, E_WARNING, "Length is too large to safely generate");
 		return FAILURE;
 	}
 
@@ -124,47 +123,20 @@ static int php_password_make_salt(size_t length, char *ret TSRMLS_DC) /* {{{ */
 
 	buffer = (char *) safe_emalloc(raw_length, 1, 1);
 
-#if PHP_WIN32
-	{
-		BYTE *iv_b = (BYTE *) buffer;
-		if (php_win32_get_random_bytes(iv_b, raw_length) == SUCCESS) {
-			buffer_valid = 1;
-		}
-	}
-#else
-	{
-		int fd, n;
-		size_t read_bytes = 0;
-		fd = open("/dev/urandom", O_RDONLY);
-		if (fd >= 0) {
-			while (read_bytes < raw_length) {
-				n = read(fd, buffer + read_bytes, raw_length - read_bytes);
-				if (n < 0) {
-					break;
-				}
-				read_bytes += (size_t) n;
-			}
-			close(fd);
-		}
-		if (read_bytes >= raw_length) {
-			buffer_valid = 1;
-		}
-	}
-#endif
-	if (!buffer_valid) {
-		for (i = 0; i < raw_length; i++) {
-			buffer[i] ^= (char) (255.0 * php_rand(TSRMLS_C) / RAND_MAX);
-		}
+	if (FAILURE == php_random_bytes_silent(buffer, raw_length)) {
+		php_error_docref(NULL, E_WARNING, "Unable to generate salt");
+		efree(buffer);
+		return FAILURE;
 	}
 
-	result = safe_emalloc(length, 1, 1); 
+	result = safe_emalloc(length, 1, 1);
 	if (php_password_salt_to64(buffer, raw_length, length, result) == FAILURE) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Generated salt too short");
+		php_error_docref(NULL, E_WARNING, "Generated salt too short");
 		efree(buffer);
 		efree(result);
 		return FAILURE;
 	}
-	memcpy(ret, result, (int) length);
+	memcpy(ret, result, length);
 	efree(result);
 	efree(buffer);
 	ret[length] = 0;
@@ -175,31 +147,25 @@ static int php_password_make_salt(size_t length, char *ret TSRMLS_DC) /* {{{ */
 PHP_FUNCTION(password_get_info)
 {
 	php_password_algo algo;
-	int hash_len;
+	size_t hash_len;
 	char *hash, *algo_name;
-	zval *options;
+	zval options;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "s", &hash, &hash_len) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "s", &hash, &hash_len) == FAILURE) {
 		return;
 	}
 
-	if (hash_len < 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Supplied password hash too long to safely identify");
-		RETURN_FALSE;
-	}
-
-	ALLOC_INIT_ZVAL(options);
-	array_init(options);
+	array_init(&options);
 
 	algo = php_password_determine_algo(hash, (size_t) hash_len);
 	algo_name = php_password_get_algo_name(algo);
-	
+
 	switch (algo) {
 		case PHP_PASSWORD_BCRYPT:
 			{
-				long cost = PHP_PASSWORD_BCRYPT_COST;
-				sscanf(hash, "$2y$%ld$", &cost);
-				add_assoc_long(options, "cost", cost);
+				zend_long cost = PHP_PASSWORD_BCRYPT_COST;
+				sscanf(hash, "$2y$" ZEND_LONG_FMT "$", &cost);
+				add_assoc_long(&options, "cost", cost);
 			}
 			break;
 		case PHP_PASSWORD_UNKNOWN:
@@ -208,32 +174,27 @@ PHP_FUNCTION(password_get_info)
 	}
 
 	array_init(return_value);
-	
+
 	add_assoc_long(return_value, "algo", algo);
-	add_assoc_string(return_value, "algoName", algo_name, 1);
-	add_assoc_zval(return_value, "options", options);	
+	add_assoc_string(return_value, "algoName", algo_name);
+	add_assoc_zval(return_value, "options", &options);
 }
 
 PHP_FUNCTION(password_needs_rehash)
 {
-	long new_algo = 0;
+	zend_long new_algo = 0;
 	php_password_algo algo;
-	int hash_len;
+	size_t hash_len;
 	char *hash;
 	HashTable *options = 0;
-	zval **option_buffer;
-	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl|H", &hash, &hash_len, &new_algo, &options) == FAILURE) {
+	zval *option_buffer;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sl|H", &hash, &hash_len, &new_algo, &options) == FAILURE) {
 		return;
 	}
 
-	if (hash_len < 0) {
-		php_error_docref(NULL TSRMLS_CC, E_WARNING, "Supplied password hash too long to safely identify");
-		RETURN_FALSE;
-	}
-
 	algo = php_password_determine_algo(hash, (size_t) hash_len);
-	
+
 	if (algo != new_algo) {
 		RETURN_TRUE;
 	}
@@ -241,21 +202,13 @@ PHP_FUNCTION(password_needs_rehash)
 	switch (algo) {
 		case PHP_PASSWORD_BCRYPT:
 			{
-				long new_cost = PHP_PASSWORD_BCRYPT_COST, cost = 0;
-				
-				if (options && zend_symtable_find(options, "cost", sizeof("cost"), (void **) &option_buffer) == SUCCESS) {
-					if (Z_TYPE_PP(option_buffer) != IS_LONG) {
-						zval cast_option_buffer;
-						MAKE_COPY_ZVAL(option_buffer, &cast_option_buffer);
-						convert_to_long(&cast_option_buffer);
-						new_cost = Z_LVAL(cast_option_buffer);
-						zval_dtor(&cast_option_buffer);
-					} else {
-						new_cost = Z_LVAL_PP(option_buffer);
-					}
+				zend_long new_cost = PHP_PASSWORD_BCRYPT_COST, cost = 0;
+
+				if (options && (option_buffer = zend_hash_str_find(options, "cost", sizeof("cost")-1)) != NULL) {
+					new_cost = zval_get_long(option_buffer);
 				}
 
-				sscanf(hash, "$2y$%ld$", &cost);
+				sscanf(hash, "$2y$" ZEND_LONG_FMT "$", &cost);
 				if (cost != new_cost) {
 					RETURN_TRUE;
 				}
@@ -273,33 +226,34 @@ Verify a hash created using crypt() or password_hash() */
 PHP_FUNCTION(password_verify)
 {
 	int status = 0, i;
-	int password_len, hash_len;
-	char *ret, *password, *hash;
-	
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "ss", &password, &password_len, &hash, &hash_len) == FAILURE) {
+	size_t password_len, hash_len;
+	char *password, *hash;
+	zend_string *ret;
+
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "ss", &password, &password_len, &hash, &hash_len) == FAILURE) {
 		RETURN_FALSE;
 	}
-	if (php_crypt(password, password_len, hash, hash_len, &ret) == FAILURE) {
+	if ((ret = php_crypt(password, (int)password_len, hash, (int)hash_len, 1)) == NULL) {
 		RETURN_FALSE;
 	}
 
-	if (strlen(ret) != hash_len || hash_len < 13) {
-		efree(ret);
+	if (ZSTR_LEN(ret) != hash_len || hash_len < 13) {
+		zend_string_free(ret);
 		RETURN_FALSE;
 	}
-	
+
 	/* We're using this method instead of == in order to provide
-	 * resistence towards timing attacks. This is a constant time
+	 * resistance towards timing attacks. This is a constant time
 	 * equality check that will always check every byte of both
 	 * values. */
 	for (i = 0; i < hash_len; i++) {
-		status |= (ret[i] ^ hash[i]);
+		status |= (ZSTR_VAL(ret)[i] ^ hash[i]);
 	}
 
-	efree(ret);
+	zend_string_free(ret);
 
 	RETURN_BOOL(status == 0);
-	
+
 }
 /* }}} */
 
@@ -307,145 +261,126 @@ PHP_FUNCTION(password_verify)
 Hash a password */
 PHP_FUNCTION(password_hash)
 {
-	char *hash_format, *hash, *salt, *password, *result;
-	long algo = 0;
-	int password_len = 0, hash_len;
+	char hash_format[8], *hash, *salt, *password;
+	zend_long algo = 0;
+	size_t password_len = 0;
+	int hash_len;
 	size_t salt_len = 0, required_salt_len = 0, hash_format_len;
 	HashTable *options = 0;
-	zval **option_buffer;
+	zval *option_buffer;
+	zend_string *result;
 
-	if (zend_parse_parameters(ZEND_NUM_ARGS() TSRMLS_CC, "sl|H", &password, &password_len, &algo, &options) == FAILURE) {
+	if (zend_parse_parameters(ZEND_NUM_ARGS(), "sl|H", &password, &password_len, &algo, &options) == FAILURE) {
 		return;
 	}
 
 	switch (algo) {
 		case PHP_PASSWORD_BCRYPT:
 		{
-			long cost = PHP_PASSWORD_BCRYPT_COST;
-	
-			if (options && zend_symtable_find(options, "cost", 5, (void **) &option_buffer) == SUCCESS) {
-				if (Z_TYPE_PP(option_buffer) != IS_LONG) {
-					zval cast_option_buffer;
-					MAKE_COPY_ZVAL(option_buffer, &cast_option_buffer);
-					convert_to_long(&cast_option_buffer);
-					cost = Z_LVAL(cast_option_buffer);
-					zval_dtor(&cast_option_buffer);
-				} else {
-					cost = Z_LVAL_PP(option_buffer);
-				}
+			zend_long cost = PHP_PASSWORD_BCRYPT_COST;
+
+			if (options && (option_buffer = zend_hash_str_find(options, "cost", sizeof("cost")-1)) != NULL) {
+				cost = zval_get_long(option_buffer);
 			}
-	
+
 			if (cost < 4 || cost > 31) {
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Invalid bcrypt cost parameter specified: %ld", cost);
+				php_error_docref(NULL, E_WARNING, "Invalid bcrypt cost parameter specified: " ZEND_LONG_FMT, cost);
 				RETURN_NULL();
 			}
-			
+
 			required_salt_len = 22;
-			hash_format = emalloc(8);
-			sprintf(hash_format, "$2y$%02ld$", cost);
+			sprintf(hash_format, "$2y$%02ld$", (long) cost);
 			hash_format_len = 7;
 		}
 		break;
 		case PHP_PASSWORD_UNKNOWN:
 		default:
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Unknown password hashing algorithm: %ld", algo);
+			php_error_docref(NULL, E_WARNING, "Unknown password hashing algorithm: " ZEND_LONG_FMT, algo);
 			RETURN_NULL();
 	}
 
-	if (options && zend_symtable_find(options, "salt", 5, (void**) &option_buffer) == SUCCESS) {
-		char *buffer;
-		int buffer_len_int = 0;
-		size_t buffer_len;
-		switch (Z_TYPE_PP(option_buffer)) {
+	if (options && (option_buffer = zend_hash_str_find(options, "salt", sizeof("salt")-1)) != NULL) {
+		zend_string *buffer;
+
+		php_error_docref(NULL, E_DEPRECATED, "Use of the 'salt' option to password_hash is deprecated");
+
+		switch (Z_TYPE_P(option_buffer)) {
 			case IS_STRING:
-				buffer = estrndup(Z_STRVAL_PP(option_buffer), Z_STRLEN_PP(option_buffer));
-				buffer_len_int = Z_STRLEN_PP(option_buffer);
+				buffer = zend_string_copy(Z_STR_P(option_buffer));
 				break;
 			case IS_LONG:
 			case IS_DOUBLE:
-			case IS_OBJECT: {
-				zval cast_option_buffer;
-				MAKE_COPY_ZVAL(option_buffer, &cast_option_buffer);
-				convert_to_string(&cast_option_buffer);
-				if (Z_TYPE(cast_option_buffer) == IS_STRING) {
-					buffer = estrndup(Z_STRVAL(cast_option_buffer), Z_STRLEN(cast_option_buffer));
-					buffer_len_int = Z_STRLEN(cast_option_buffer);
-					zval_dtor(&cast_option_buffer);
-					break;
-				}
-				zval_dtor(&cast_option_buffer);
-			}
-			case IS_BOOL:
+			case IS_OBJECT:
+				buffer = zval_get_string(option_buffer);
+				break;
+			case IS_FALSE:
+			case IS_TRUE:
 			case IS_NULL:
 			case IS_RESOURCE:
 			case IS_ARRAY:
 			default:
-				efree(hash_format);
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Non-string salt parameter supplied");
+				php_error_docref(NULL, E_WARNING, "Non-string salt parameter supplied");
 				RETURN_NULL();
 		}
-		if (buffer_len_int < 0) {
-			efree(hash_format);
-			efree(buffer);
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Supplied salt is too long");
-		}
-		buffer_len = (size_t) buffer_len_int;
-		if (buffer_len < required_salt_len) {
-			efree(hash_format);
-			efree(buffer);
-			php_error_docref(NULL TSRMLS_CC, E_WARNING, "Provided salt is too short: %lu expecting %lu", (unsigned long) buffer_len, (unsigned long) required_salt_len);
+
+		/* XXX all the crypt related APIs work with int for string length.
+			That should be revised for size_t and then we maybe don't require
+			the > INT_MAX check. */
+		if (ZSTR_LEN(buffer) > INT_MAX) {
+			php_error_docref(NULL, E_WARNING, "Supplied salt is too long");
 			RETURN_NULL();
-		} else if (php_password_salt_is_alphabet(buffer, buffer_len) == FAILURE) {
+		} else if (ZSTR_LEN(buffer) < required_salt_len) {
+			php_error_docref(NULL, E_WARNING, "Provided salt is too short: %zd expecting %zd", ZSTR_LEN(buffer), required_salt_len);
+			zend_string_release(buffer);
+			RETURN_NULL();
+		} else if (php_password_salt_is_alphabet(ZSTR_VAL(buffer), ZSTR_LEN(buffer)) == FAILURE) {
 			salt = safe_emalloc(required_salt_len, 1, 1);
-			if (php_password_salt_to64(buffer, buffer_len, required_salt_len, salt) == FAILURE) {
-				efree(hash_format);
-				efree(buffer);
+			if (php_password_salt_to64(ZSTR_VAL(buffer), ZSTR_LEN(buffer), required_salt_len, salt) == FAILURE) {
 				efree(salt);
-				php_error_docref(NULL TSRMLS_CC, E_WARNING, "Provided salt is too short: %lu", (unsigned long) buffer_len);
+				php_error_docref(NULL, E_WARNING, "Provided salt is too short: %zd", ZSTR_LEN(buffer));
+				zend_string_release(buffer);
 				RETURN_NULL();
 			}
 			salt_len = required_salt_len;
 		} else {
 			salt = safe_emalloc(required_salt_len, 1, 1);
-			memcpy(salt, buffer, (int) required_salt_len);
+			memcpy(salt, ZSTR_VAL(buffer), required_salt_len);
 			salt_len = required_salt_len;
 		}
-		efree(buffer);
+		zend_string_release(buffer);
 	} else {
 		salt = safe_emalloc(required_salt_len, 1, 1);
-		if (php_password_make_salt(required_salt_len, salt TSRMLS_CC) == FAILURE) {
-			efree(hash_format);
+		if (php_password_make_salt(required_salt_len, salt) == FAILURE) {
 			efree(salt);
 			RETURN_FALSE;
 		}
 		salt_len = required_salt_len;
 	}
-	
+
 	salt[salt_len] = 0;
 
 	hash = safe_emalloc(salt_len + hash_format_len, 1, 1);
 	sprintf(hash, "%s%s", hash_format, salt);
 	hash[hash_format_len + salt_len] = 0;
 
-	efree(hash_format);
 	efree(salt);
 
 	/* This cast is safe, since both values are defined here in code and cannot overflow */
 	hash_len = (int) (hash_format_len + salt_len);
 
-	if (php_crypt(password, password_len, hash, hash_len, &result) == FAILURE) {
+	if ((result = php_crypt(password, (int)password_len, hash, hash_len, 1)) == NULL) {
 		efree(hash);
 		RETURN_FALSE;
 	}
 
 	efree(hash);
 
-	if (strlen(result) < 13) {
-		efree(result);
+	if (ZSTR_LEN(result) < 13) {
+		zend_string_free(result);
 		RETURN_FALSE;
 	}
 
-	RETURN_STRING(result, 0);
+	RETURN_STR(result);
 }
 /* }}} */
 
