@@ -1,6 +1,6 @@
 /*
    +----------------------------------------------------------------------+
-   | PHP Version 5                                                        |
+   | PHP Version 7                                                        |
    +----------------------------------------------------------------------+
    | Copyright (c) 1997-2016 The PHP Group                                |
    +----------------------------------------------------------------------+
@@ -148,11 +148,7 @@ static void sapi_lsapi_ini_defaults(HashTable *configuration_hash)
 
 /* {{{ sapi_lsapi_ub_write
  */
-#if PHP_MAJOR_VERSION >= 7
-static size_t sapi_lsapi_ub_write(const char *str, size_t str_length TSRMLS_DC)
-#else
-static int sapi_lsapi_ub_write(const char *str, uint str_length TSRMLS_DC)
-#endif
+static size_t sapi_lsapi_ub_write(const char *str, size_t str_length)
 {
     int ret;
     int remain;
@@ -181,7 +177,7 @@ static int sapi_lsapi_ub_write(const char *str, uint str_length TSRMLS_DC)
 
 /* {{{ sapi_lsapi_flush
  */
-static void sapi_lsapi_flush(void * server_context)
+static void sapi_lsapi_flush( void * server_context )
 {
     if ( lsapi_mode ) {
         if ( LSAPI_Flush() == -1) {
@@ -194,11 +190,12 @@ static void sapi_lsapi_flush(void * server_context)
 
 /* {{{ sapi_lsapi_deactivate
  */
-static int sapi_lsapi_deactivate(TSRMLS_D)
+static int sapi_lsapi_deactivate(void)
 {
     if ( SG(request_info).path_translated )
     {
         efree( SG(request_info).path_translated );
+        SG(request_info).path_translated = NULL;
     }
 
     return SUCCESS;
@@ -210,7 +207,7 @@ static int sapi_lsapi_deactivate(TSRMLS_D)
 
 /* {{{ sapi_lsapi_getenv
  */
-static char *sapi_lsapi_getenv( char * name, size_t name_len TSRMLS_DC )
+static char *sapi_lsapi_getenv( char * name, size_t name_len )
 {
     if ( lsapi_mode ) {
         return LSAPI_GetEnv( name );
@@ -221,70 +218,27 @@ static char *sapi_lsapi_getenv( char * name, size_t name_len TSRMLS_DC )
 /* }}} */
 
 
-#if PHP_MAJOR_VERSION > 4
-
 static int add_variable( const char * pKey, int keyLen, const char * pValue, int valLen,
-                         void * arg TSRMLS_DC)
+                         void * arg )
 {
-#if PHP_MAJOR_VERSION >= 7
     int filter_arg = (Z_ARR_P((zval *)arg) == Z_ARR(PG(http_globals)[TRACK_VARS_ENV]))
         ? PARSE_ENV : PARSE_SERVER;
+    char * new_val = (char *) pValue;
     size_t new_val_len;
-#else
-    int filter_arg = (arg == PG(http_globals)[TRACK_VARS_ENV])?PARSE_ENV:PARSE_SERVER;
-    unsigned int new_val_len;
-#endif
 
-    char * new_val = (char *) pValue; 
-
-    if (sapi_module.input_filter(filter_arg, (char *)pKey, &new_val, valLen, &new_val_len TSRMLS_CC)) {
-        php_register_variable_safe((char *)pKey, new_val, new_val_len, (zval *)arg TSRMLS_CC);
+    if (sapi_module.input_filter(filter_arg, (char *)pKey, &new_val, valLen, &new_val_len)) {
+        php_register_variable_safe((char *)pKey, new_val, new_val_len, (zval *)arg );
     }
     return 1;
 }
 
-#else
-
-static int add_variable( const char * pKey, int keyLen, const char * pValue, int valLen,
-                         void * arg )
-{
-    zval * gpc_element, **gpc_element_p;
-    HashTable * symtable1 = Z_ARRVAL_P((zval * )arg);
-    register char * pKey1 = (char *)pKey;
-
-    MAKE_STD_ZVAL(gpc_element);
-    Z_STRLEN_P( gpc_element ) = valLen;
-    Z_STRVAL_P( gpc_element ) = estrndup(pValue, valLen);
-    Z_TYPE_P( gpc_element ) = IS_STRING;
-    zend_hash_update( symtable1, pKey1, keyLen + 1, &gpc_element, sizeof( zval *), (void **) &gpc_element_p );
-    return 1;
-}
-#endif
-
-
-#if defined(ZTS) && PHP_MAJOR_VERSION < 7
-struct param_zts
-{
-    void *arg;
-    TSRMLS_D ;
-};
-
-static int add_variable_zts(const char * pKey, int keyLen, const char * pValue, int valLen,
-                         void * arg )
-{
-    struct param_zts * params = (struct param_zts *)arg;
-    return add_variable(pKey, keyLen, pValue, valLen, params->arg, params->TSRMLS_C);
-}
-#endif
-
-static void litespeed_php_import_environment_variables(zval *array_ptr TSRMLS_DC)
+static void litespeed_php_import_environment_variables(zval *array_ptr)
 {
 	char buf[128];
 	char **env, *p, *t = buf;
 	size_t alloc_size = sizeof(buf);
 	unsigned long nlen; /* ptrdiff_t is not portable */
 
-#if PHP_MAJOR_VERSION >= 7
     if (Z_TYPE(PG(http_globals)[TRACK_VARS_ENV]) == IS_ARRAY &&
         Z_ARR_P(array_ptr) != Z_ARR(PG(http_globals)[TRACK_VARS_ENV]) &&
         zend_hash_num_elements(Z_ARRVAL(PG(http_globals)[TRACK_VARS_ENV])) > 0
@@ -300,29 +254,6 @@ static void litespeed_php_import_environment_variables(zval *array_ptr TSRMLS_DC
         ZVAL_DUP(array_ptr, &PG(http_globals)[TRACK_VARS_SERVER]);
 		return;
 	}
-#else
-    if (PG(http_globals)[TRACK_VARS_ENV] &&
-        array_ptr != PG(http_globals)[TRACK_VARS_ENV] &&
-        Z_TYPE_P(PG(http_globals)[TRACK_VARS_ENV]) == IS_ARRAY &&
-        zend_hash_num_elements(Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_ENV])) > 0
-    ) {
-        zval_dtor(array_ptr);
-        *array_ptr = *PG(http_globals)[TRACK_VARS_ENV];
-        INIT_PZVAL(array_ptr);
-        zval_copy_ctor(array_ptr);
-        return;
-    } else if (PG(http_globals)[TRACK_VARS_SERVER] &&
-        array_ptr != PG(http_globals)[TRACK_VARS_SERVER] &&
-        Z_TYPE_P(PG(http_globals)[TRACK_VARS_SERVER]) == IS_ARRAY &&
-        zend_hash_num_elements(Z_ARRVAL_P(PG(http_globals)[TRACK_VARS_SERVER])) > 0
-    ) {
-        zval_dtor(array_ptr);
-        *array_ptr = *PG(http_globals)[TRACK_VARS_SERVER];
-        INIT_PZVAL(array_ptr);
-        zval_copy_ctor(array_ptr);
-        return;
-    }
-#endif
 
 	for (env = environ; env != NULL && *env != NULL; env++) {
 		p = strchr(*env, '=');
@@ -336,79 +267,35 @@ static void litespeed_php_import_environment_variables(zval *array_ptr TSRMLS_DC
 		}
 		memcpy(t, *env, nlen);
 		t[nlen] = '\0';
-		add_variable(t, nlen, p + 1, strlen( p + 1 ), array_ptr TSRMLS_CC);
+		add_variable(t, nlen, p + 1, strlen( p + 1 ), array_ptr);
 	}
 	if (t != buf && t != NULL) {
 		efree(t);
 	}
 }
 
-
-#if ((PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION < 4) || PHP_MAJOR_VERSION < 5)
-static int add_variable_magic_quote( const char * pKey, int keyLen, const char * pValue, int valLen, 
-                         void * arg )
-{
-    zval * gpc_element, **gpc_element_p;
-    HashTable * symtable1 = Z_ARRVAL_P((zval * )arg);
-    register char * pKey1 = (char *)pKey;
-
-    MAKE_STD_ZVAL(gpc_element);
-    Z_STRLEN_P( gpc_element ) = valLen;
-    Z_STRVAL_P( gpc_element ) = php_addslashes((char *)pValue, valLen, &Z_STRLEN_P( gpc_element ), 0 );
-    Z_TYPE_P( gpc_element ) = IS_STRING;
-#if PHP_MAJOR_VERSION > 4
-    zend_symtable_update( symtable1, pKey1, keyLen + 1, &gpc_element, sizeof( zval *), (void **) &gpc_element_p );
-#else
-    zend_hash_update( symtable1, pKey1, keyLen + 1, &gpc_element, sizeof( zval *), (void **) &gpc_element_p );
-#endif
-    return 1;
-}
-
-#endif
-
 /* {{{ sapi_lsapi_register_variables
  */
-static void sapi_lsapi_register_variables(zval *track_vars_array TSRMLS_DC)
+static void sapi_lsapi_register_variables(zval *track_vars_array)
 {
     char * php_self = "";
-#if defined(ZTS) && PHP_MAJOR_VERSION < 7
-    struct param_zts params;
-    params.arg = track_vars_array;
-    params.TSRMLS_C = TSRMLS_C;
-#endif
     if ( lsapi_mode ) {
         if ( (SG(request_info).request_uri ) )
             php_self = (SG(request_info).request_uri );
 
-        litespeed_php_import_environment_variables(track_vars_array TSRMLS_CC);
+        litespeed_php_import_environment_variables(track_vars_array);
 
-#if ((PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION < 4) || PHP_MAJOR_VERSION < 5)
-        if (!PG(magic_quotes_gpc)) {
-#endif
-#if defined(ZTS) && PHP_MAJOR_VERSION < 7
-            LSAPI_ForeachHeader( add_variable_zts, &params );
-            LSAPI_ForeachEnv( add_variable_zts, &params );
-#else
-            LSAPI_ForeachHeader( add_variable, track_vars_array );
-            LSAPI_ForeachEnv( add_variable, track_vars_array );
-#endif            
-            add_variable("PHP_SELF", 8, php_self, strlen( php_self ), 
-                         track_vars_array TSRMLS_CC);
-#if ((PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION < 4) || PHP_MAJOR_VERSION < 5)
-        } else {
-            LSAPI_ForeachHeader( add_variable_magic_quote, track_vars_array );
-            LSAPI_ForeachEnv( add_variable_magic_quote, track_vars_array );
-            add_variable_magic_quote("PHP_SELF", 8, php_self, strlen( php_self ), track_vars_array );
-        }
-#endif
+		LSAPI_ForeachHeader( add_variable, track_vars_array );
+		LSAPI_ForeachEnv( add_variable, track_vars_array );
+		add_variable("PHP_SELF", 8, php_self, strlen( php_self ), track_vars_array );
     } else {
-        php_import_environment_variables(track_vars_array TSRMLS_CC);
+        php_import_environment_variables(track_vars_array);
 
-        php_register_variable("PHP_SELF", php_self, track_vars_array TSRMLS_CC);
-        php_register_variable("SCRIPT_NAME", php_self, track_vars_array TSRMLS_CC);
-        php_register_variable("SCRIPT_FILENAME", script_filename, track_vars_array TSRMLS_CC);
-        php_register_variable("PATH_TRANSLATED", script_filename, track_vars_array TSRMLS_CC);
-        php_register_variable("DOCUMENT_ROOT", "", track_vars_array TSRMLS_CC);
+        php_register_variable("PHP_SELF", php_self, track_vars_array);
+        php_register_variable("SCRIPT_NAME", php_self, track_vars_array);
+        php_register_variable("SCRIPT_FILENAME", script_filename, track_vars_array);
+        php_register_variable("PATH_TRANSLATED", script_filename, track_vars_array);
+        php_register_variable("DOCUMENT_ROOT", "", track_vars_array);
 
     }
 }
@@ -417,11 +304,7 @@ static void sapi_lsapi_register_variables(zval *track_vars_array TSRMLS_DC)
 
 /* {{{ sapi_lsapi_read_post
  */
-#if PHP_MAJOR_VERSION >= 7
-static size_t sapi_lsapi_read_post(char *buffer, size_t count_bytes TSRMLS_DC)
-#else
-static int sapi_lsapi_read_post(char *buffer, uint count_bytes TSRMLS_DC)
-#endif
+static size_t sapi_lsapi_read_post(char *buffer, size_t count_bytes)
 {
     if ( lsapi_mode ) {
         return LSAPI_ReadReqBody( buffer, (unsigned long long)count_bytes );
@@ -436,7 +319,7 @@ static int sapi_lsapi_read_post(char *buffer, uint count_bytes TSRMLS_DC)
 
 /* {{{ sapi_lsapi_read_cookies
  */
-static char *sapi_lsapi_read_cookies(TSRMLS_D)
+static char *sapi_lsapi_read_cookies(void)
 {
     if ( lsapi_mode ) {
         return LSAPI_GetHeader( H_COOKIE );
@@ -449,7 +332,7 @@ static char *sapi_lsapi_read_cookies(TSRMLS_D)
 
 /* {{{ sapi_lsapi_send_headers
  */
-static int sapi_lsapi_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
+static int sapi_lsapi_send_headers(sapi_headers_struct *sapi_headers)
 {
     sapi_header_struct  *h;
     zend_llist_position pos;
@@ -468,7 +351,7 @@ static int sapi_lsapi_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
             int     len;
             char    headerBuf[SAPI_LSAPI_MAX_HEADER_LENGTH];
 
-            hd = sapi_get_default_content_type(TSRMLS_C);
+            hd = sapi_get_default_content_type();
             len = snprintf( headerBuf, SAPI_LSAPI_MAX_HEADER_LENGTH - 1,
                             "Content-type: %s", hd );
             efree(hd);
@@ -486,11 +369,7 @@ static int sapi_lsapi_send_headers(sapi_headers_struct *sapi_headers TSRMLS_DC)
 
 /* {{{ sapi_lsapi_send_headers
  */
-static void sapi_lsapi_log_message(char *message
-#if PHP_MAJOR_VERSION > 7 || (PHP_MAJOR_VERSION == 7 && PHP_MINOR_VERSION >= 1)
-                                   , int syslog_type_int   /* unused */
-#endif
-                                   TSRMLS_DC)
+static void sapi_lsapi_log_message(char *message)
 {
     char buf[8192];
     int len = strlen( message );
@@ -555,10 +434,6 @@ static sapi_module_struct lsapi_sapi_module =
     sapi_lsapi_register_variables,  /* register server variables */
     sapi_lsapi_log_message,         /* Log message */
 
-#if PHP_MAJOR_VERSION > 7 || (PHP_MAJOR_VERSION == 7 && PHP_MINOR_VERSION >= 1)
-    NULL,                           /* Get request time */
-    NULL,                           /* Child terminate */
-#else
     NULL,                           /* php.ini path override */
     NULL,                           /* block interruptions */
     NULL,                           /* unblock interruptions */
@@ -567,18 +442,17 @@ static sapi_module_struct lsapi_sapi_module =
     NULL,                           /* executable location */
 
     0,                              /* php.ini ignore */
-#endif
 
     STANDARD_SAPI_MODULE_PROPERTIES
 
 };
 /* }}} */
 
-static void init_request_info( TSRMLS_D )
+static void init_request_info( void )
 {
     char * pContentType = LSAPI_GetHeader( H_CONTENT_TYPE );
     char * pAuth;
-    
+
     SG(request_info).content_type = pContentType ? pContentType : "";
     SG(request_info).request_method = LSAPI_GetRequestMethod();
     SG(request_info).query_string = LSAPI_GetQueryString();
@@ -588,9 +462,9 @@ static void init_request_info( TSRMLS_D )
 
     /* It is not reset by zend engine, set it to 200. */
     SG(sapi_headers).http_response_code = 200;
-    
+
     pAuth = LSAPI_GetHeader( H_AUTHORIZATION );
-    php_handle_auth_data(pAuth TSRMLS_CC);
+    php_handle_auth_data(pAuth);
 }
 
 static char s_cur_chdir[4096] = "";
@@ -617,7 +491,7 @@ static int lsapi_chdir_primary_script( zend_file_handle * file_handle )
         if ( !CWDG(cwd).cwd ||
              ( strcmp( file_handle->filename, CWDG(cwd).cwd ) != 0 ) ) {
             CWDG(cwd).cwd_length = p - file_handle->filename;
-            CWDG(cwd).cwd = (char *) realloc(CWDG(cwd).cwd, CWDG(cwd).cwd_length+1);            
+            CWDG(cwd).cwd = (char *) realloc(CWDG(cwd).cwd, CWDG(cwd).cwd_length+1);
             memmove( CWDG(cwd).cwd, file_handle->filename, CWDG(cwd).cwd_length+1 );
         }
         *p = ch;
@@ -649,7 +523,7 @@ static int lsapi_fopen_primary_script( zend_file_handle * file_handle )
     return 0;
 }
 
-static int lsapi_execute_script( zend_file_handle * file_handle TSRMLS_DC)
+static int lsapi_execute_script( zend_file_handle * file_handle)
 {
     char *p;
     int len;
@@ -668,33 +542,32 @@ static int lsapi_execute_script( zend_file_handle * file_handle TSRMLS_DC)
         len = 0;
     memccpy( p, SG(request_info).path_translated + len, 0, 46 );
 
-    php_execute_script(file_handle TSRMLS_CC);
+    php_execute_script(file_handle);
     return 0;
 
 }
 
-static int lsapi_activate_user_ini(TSRMLS_D);
+static int lsapi_activate_user_ini();
 
-static int lsapi_module_main(int show_source TSRMLS_DC)
+static int lsapi_module_main(int show_source)
 {
     zend_file_handle file_handle = {0};
 
-    if (php_request_startup(TSRMLS_C) == FAILURE ) {
+    if (php_request_startup() == FAILURE ) {
         return -1;
     }
     
-#if PHP_MAJOR_VERSION > 5 || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3)
-    if (parse_user_ini && lsapi_activate_user_ini(TSRMLS_C) == FAILURE) {
+    if (parse_user_ini && lsapi_activate_user_ini() == FAILURE) {
         return -1;
     }
-#endif    
+
     if (show_source) {
         zend_syntax_highlighter_ini syntax_highlighter_ini;
 
         php_get_highlight_struct(&syntax_highlighter_ini);
-        highlight_file(SG(request_info).path_translated, &syntax_highlighter_ini TSRMLS_CC);
+        highlight_file(SG(request_info).path_translated, &syntax_highlighter_ini);
     } else {
-        lsapi_execute_script( &file_handle TSRMLS_CC);
+        lsapi_execute_script( &file_handle);
     }
     zend_try {
         php_request_shutdown(NULL);
@@ -708,7 +581,7 @@ static int alter_ini( const char * pKey, int keyLen, const char * pValue, int va
                 void * arg )
 {
 #if PHP_MAJOR_VERSION >= 7
-	zend_string * psKey; 
+    zend_string * psKey;
 #endif
     int type = ZEND_INI_PERDIR;
     if ( '\001' == *pKey ) {
@@ -727,36 +600,24 @@ static int alter_ini( const char * pKey, int keyLen, const char * pValue, int va
         {
 #if PHP_MAJOR_VERSION >= 7
             --keyLen;
-            psKey = zend_string_init( pKey, keyLen, 1 );
-            zend_alter_ini_entry_chars(psKey, 
+            psKey = zend_string_init(pKey, keyLen, 1);
+            zend_alter_ini_entry_chars(psKey,
                              (char *)pValue, valLen,
                              type, PHP_INI_STAGE_ACTIVATE);
-            zend_string_release( psKey );
+            zend_string_release(psKey);
 #else
             zend_alter_ini_entry((char *)pKey, keyLen,
                              (char *)pValue, valLen,
-                             type, PHP_INI_STAGE_ACTIVATE);            
+                             type, PHP_INI_STAGE_ACTIVATE);
 #endif
         }
     }
     return 1;
 }
 
-static void user_config_cache_entry_dtor(
-#if PHP_MAJOR_VERSION >= 7
-                                         zval
-#else
-                                         void
-#endif
-                                               *el)
+static void user_config_cache_entry_dtor(zval *el)
 {
-    user_config_cache_entry *entry =
-#if PHP_MAJOR_VERSION >= 7
-                                      (user_config_cache_entry *)Z_PTR_P(el)
-#else
-                                     *(user_config_cache_entry **)el
-#endif
-    ;
+    user_config_cache_entry *entry = (user_config_cache_entry *)Z_PTR_P(el);
     zend_hash_destroy(&entry->user_config);
     free(entry);
 }
@@ -830,16 +691,12 @@ static void walk_down_the_path(char* path_start,
     }
 }
 
-#if PHP_MAJOR_VERSION > 5 || (PHP_MAJOR_VERSION == 5 && PHP_MINOR_VERSION >= 3)
 
 typedef struct {
     char *path;
     uint path_len;
     char *doc_root;
     user_config_cache_entry *entry;
-#if defined(ZTS) && PHP_MAJOR_VERSION < 7
-    TSRMLS_D ;
-#endif
 } _lsapi_activate_user_ini_ctx;
 
 typedef int (*fn_activate_user_ini_chain_t)
@@ -861,11 +718,7 @@ static int lsapi_activate_user_ini_basic_checks(_lsapi_activate_user_ini_ctx *ct
         return FAILURE;
     }
 
-    ctx->doc_root = sapi_lsapi_getenv("DOCUMENT_ROOT", 0
-#if defined(ZTS) && PHP_MAJOR_VERSION < 7
-                    , ctx->TSRMLS_C
-#endif
-    );
+    ctx->doc_root = sapi_lsapi_getenv("DOCUMENT_ROOT", 0);
     DEBUG_MESSAGE("doc_root: %s", ctx->doc_root);
 
     if (*fn_next) {
@@ -906,11 +759,7 @@ static int lsapi_activate_user_ini_mk_realpath(_lsapi_activate_user_ini_ctx *ctx
     fn_activate_user_ini_chain_t *fn_next = next;
 
     if (!IS_ABSOLUTE_PATH(ctx->path, ctx->path_len)) {
-        real_path = tsrm_realpath(ctx->path, NULL
-#if defined(ZTS) && PHP_MAJOR_VERSION < 7
-                    , ctx->TSRMLS_C
-#endif
-        );
+        real_path = tsrm_realpath(ctx->path, NULL);
         if (!real_path) {
             return SUCCESS;
         }
@@ -937,18 +786,7 @@ static int lsapi_activate_user_ini_mk_user_config(_lsapi_activate_user_ini_ctx *
     fn_activate_user_ini_chain_t *fn_next = next;
 
     /* Find cached config entry: If not found, create one */
-#if PHP_MAJOR_VERSION >= 7
     ctx->entry = zend_hash_str_find_ptr(&user_config_cache, ctx->path, ctx->path_len);
-#else
-    {
-        user_config_cache_entry **entry_pp;
-        if (SUCCESS == zend_hash_find(&user_config_cache, ctx->path,
-                                            ctx->path_len + 1, (void **) &entry_pp))
-            ctx->entry = *entry_pp;
-        else
-            ctx->entry = NULL;
-    }
-#endif
 
     if (ctx->entry) {
         DEBUG_MESSAGE("found entry for %s", ctx->path);
@@ -957,17 +795,9 @@ static int lsapi_activate_user_ini_mk_user_config(_lsapi_activate_user_ini_ctx *
         ctx->entry = pemalloc(sizeof(user_config_cache_entry), 1);
         ctx->entry->expires = 0;
         zend_hash_init(&ctx->entry->user_config, 0, NULL,
-#if PHP_MAJOR_VERSION <= 5
-                       (dtor_func_t)
-#endif
                        config_zval_dtor, 1);
-#if PHP_MAJOR_VERSION >= 7
         zend_hash_str_update_ptr(&user_config_cache, ctx->path,
                                             ctx->path_len, ctx->entry);
-#else
-        zend_hash_update(&user_config_cache, ctx->path, ctx->path_len + 1,
-                                &ctx->entry, sizeof(&ctx->entry), NULL);
-#endif
     }
 
     if (*fn_next) {
@@ -985,22 +815,14 @@ static void walk_down_the_path_callback(char* begin,
     char tmp = end[0];
     end[0] = 0;
     DEBUG_MESSAGE("parsing %s%c%s", begin, DEFAULT_SLASH, PG(user_ini_filename));
-    php_parse_user_ini_file(begin, PG(user_ini_filename), &ctx->entry->user_config
-#if defined(ZTS) && PHP_MAJOR_VERSION < 7
-                    , ctx->TSRMLS_C
-#endif
-    );
+    php_parse_user_ini_file(begin, PG(user_ini_filename), &ctx->entry->user_config);
     end[0] = tmp;
 }
 
 static int lsapi_activate_user_ini_walk_down_the_path(_lsapi_activate_user_ini_ctx *ctx,
                                                       void* next)
 {
-    time_t request_time = sapi_get_request_time(
-#if defined(ZTS) && PHP_MAJOR_VERSION < 7
-                    ctx->TSRMLS_C
-#endif
-    );
+    time_t request_time = sapi_get_request_time();
     uint path_len, docroot_len;
     int rc = SUCCESS;
     fn_activate_user_ini_chain_t *fn_next = next;
@@ -1016,11 +838,8 @@ static int lsapi_activate_user_ini_walk_down_the_path(_lsapi_activate_user_ini_c
                 strncmp(ctx->path, ctx->doc_root, docroot_len) != 0;
 
         if (is_outside_of_docroot) {
-            php_parse_user_ini_file(ctx->path, PG(user_ini_filename), &ctx->entry->user_config
-#if defined(ZTS) && PHP_MAJOR_VERSION < 7
-                    , ctx->TSRMLS_C
-#endif
-            );
+            php_parse_user_ini_file(ctx->path, PG(user_ini_filename), 
+                                    &ctx->entry->user_config);
         } else {
             walk_down_the_path(ctx->doc_root, ctx->path,
                                &walk_down_the_path_callback, ctx);
@@ -1043,11 +862,8 @@ static int lsapi_activate_user_ini_finally(_lsapi_activate_user_ini_ctx *ctx,
     fn_activate_user_ini_chain_t *fn_next = next;
 
     DEBUG_MESSAGE("calling php_ini_activate_config()");
-    php_ini_activate_config(&ctx->entry->user_config, PHP_INI_PERDIR, PHP_INI_STAGE_HTACCESS
-#if defined(ZTS) && PHP_MAJOR_VERSION < 7
-                    , ctx->TSRMLS_C
-#endif
-    );
+    php_ini_activate_config(&ctx->entry->user_config, PHP_INI_PERDIR, 
+                            PHP_INI_STAGE_HTACCESS);
 
     if (*fn_next) {
         rc = (*fn_next)(ctx, fn_next + 1);
@@ -1078,14 +894,9 @@ static int lsapi_activate_user_ini(TSRMLS_D)
         NULL
     };
     
-#if defined(ZTS) && PHP_MAJOR_VERSION < 7
-    ctx.TSRMLS_C = TSRMLS_C;
-#endif
-
     return fn_chain[0](&ctx, (fn_activate_user_ini_chain_t*)(fn_chain + 1));
 }
 
-#endif
 
 static void override_ini()
 {
@@ -1095,7 +906,7 @@ static void override_ini()
 }
 
 
-static int processReq( TSRMLS_D )
+static int processReq( void )
 {
     int ret = 0;
     zend_first_try {
@@ -1107,9 +918,9 @@ static int processReq( TSRMLS_D )
         override_ini();
 
         if ( engine ) {
-            init_request_info( TSRMLS_C );
+            init_request_info(  );
 
-            if ( lsapi_module_main( source_highlight TSRMLS_CC ) == -1 ) {
+            if ( lsapi_module_main( source_highlight ) == -1 ) {
                 ret = -1;
             }
         } else {
@@ -1121,7 +932,7 @@ static int processReq( TSRMLS_D )
     return ret;
 }
 
-static void cli_usage( TSRMLS_D )
+static void cli_usage( void )
 {
     static const char * usage =
         "Usage: php\n"
@@ -1142,12 +953,12 @@ static void cli_usage( TSRMLS_D )
         "\n"
         "  args...    Arguments passed to script.\n";
     php_output_startup();
-    php_output_activate(TSRMLS_C);
+    php_output_activate();
     php_printf( "%s", usage );
 #ifdef PHP_OUTPUT_NEWAPI
-    php_output_end_all(TSRMLS_C);
+    php_output_end_all();
 #else
-    php_end_ob_buffers(1 TSRMLS_CC);
+    php_end_ob_buffers(1);
 #endif
 }
 
@@ -1168,7 +979,7 @@ static int parse_opt( int argc, char * argv[], int *climode,
             }
             *php_bind = strdup(*p++);
             break;
-            
+
         case 'c':
             if ( p >= argend ) {
                 fprintf( stderr, "<path> or <file> must be specified following '-c' option.\n");
@@ -1223,7 +1034,7 @@ static int cli_main( int argc, char * argv[] )
     int ret = -1;
     int c;
 #if PHP_MAJOR_VERSION >= 7
-	zend_string * psKey; 
+	zend_string * psKey;
 #endif
     lsapi_mode = 0;        /* enter CLI mode */
 
@@ -1246,11 +1057,11 @@ static int cli_main( int argc, char * argv[] )
 #endif
         for( ini = ini_defaults; *ini; ini+=2 ) {
 #if PHP_MAJOR_VERSION >= 7
-			psKey = zend_string_init( *ini, strlen( *ini ), 1 );
-            zend_alter_ini_entry_chars( psKey, 
+			psKey = zend_string_init(*ini, strlen( *ini ), 1);
+            zend_alter_ini_entry_chars(psKey,
                                 (char *)*(ini+1), strlen( *(ini+1) ),
                                 PHP_INI_SYSTEM, PHP_INI_STAGE_ACTIVATE);
-            zend_string_release( psKey );
+			zend_string_release(psKey);
 #else
             zend_alter_ini_entry( (char *)*ini, strlen( *ini )+1,
                                 (char *)*(ini+1), strlen( *(ini+1) ),
@@ -1265,28 +1076,28 @@ static int cli_main( int argc, char * argv[] )
             case 'q':
                 break;
             case 'i':
-                if (php_request_startup(TSRMLS_C) != FAILURE) {
-                    php_print_info(0xFFFFFFFF TSRMLS_CC);
+                if (php_request_startup() != FAILURE) {
+                    php_print_info(0xFFFFFFFF);
 #ifdef PHP_OUTPUT_NEWAPI
-                    php_output_end_all(TSRMLS_C);
+                    php_output_end_all();
 #else
-                    php_end_ob_buffers(1 TSRMLS_CC);
+                    php_end_ob_buffers(1);
 #endif
                     php_request_shutdown( NULL );
                     ret = 0;
                 }
                 break;
             case 'v':
-                if (php_request_startup(TSRMLS_C) != FAILURE) {
+                if (php_request_startup() != FAILURE) {
 #if ZEND_DEBUG
                     php_printf("PHP %s (%s) (built: %s %s) (DEBUG)\nCopyright (c) 1997-2016 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
 #else
                     php_printf("PHP %s (%s) (built: %s %s)\nCopyright (c) 1997-2016 The PHP Group\n%s", PHP_VERSION, sapi_module.name, __DATE__, __TIME__, get_zend_version());
 #endif
 #ifdef PHP_OUTPUT_NEWAPI
-                    php_output_end_all(TSRMLS_C);
+                    php_output_end_all();
 #else
-                    php_end_ob_buffers(1 TSRMLS_CC);
+                    php_end_ob_buffers(1);
 #endif
                     php_request_shutdown( NULL );
                     ret = 0;
@@ -1303,7 +1114,7 @@ static int cli_main( int argc, char * argv[] )
             case 'h':
             case '?':
             default:
-                cli_usage(TSRMLS_C);
+                cli_usage();
                 ret = 0;
                 break;
 
@@ -1324,32 +1135,32 @@ static int cli_main( int argc, char * argv[] )
                     SG(request_info).argc = argc - (p - argv);
                     SG(request_info).argv = p;
 
-                    if (php_request_startup(TSRMLS_C) == FAILURE ) {
+                    if (php_request_startup() == FAILURE ) {
                         fclose( file_handle.handle.fp );
                         ret = 2;
                     } else {
                         if (source_highlight == 1) {
                             zend_syntax_highlighter_ini syntax_highlighter_ini;
-                    
+
                             php_get_highlight_struct(&syntax_highlighter_ini);
-                            highlight_file(SG(request_info).path_translated, &syntax_highlighter_ini TSRMLS_CC);
+                            highlight_file(SG(request_info).path_translated, &syntax_highlighter_ini);
                         } else if (source_highlight == 2) {
                             file_handle.filename = *p;
                             file_handle.free_filename = 0;
                             file_handle.opened_path = NULL;
-                            ret = php_lint_script(&file_handle TSRMLS_CC);
+                            ret = php_lint_script(&file_handle);
                             if (ret==SUCCESS) {
                                 zend_printf("No syntax errors detected in %s\n", file_handle.filename);
                             } else {
                                 zend_printf("Errors parsing %s\n", file_handle.filename);
                             }
-                            
+
                         } else {
                             file_handle.filename = *p;
                             file_handle.free_filename = 0;
                             file_handle.opened_path = NULL;
 
-                            php_execute_script(&file_handle TSRMLS_CC);
+                            php_execute_script(&file_handle);
                             ret = EG(exit_status);
                        }
 
@@ -1359,13 +1170,13 @@ static int cli_main( int argc, char * argv[] )
                     php_printf("Could not open input file: %s.\n", *p);
                 }
             } else {
-                cli_usage(TSRMLS_C);
+                cli_usage();
             }
         }
 
     }zend_end_try();
 
-    php_module_shutdown(TSRMLS_C);
+    php_module_shutdown();
 
 #ifdef ZTS
     tsrm_shutdown();
@@ -1421,7 +1232,7 @@ void start_children( int children )
                 running++;
                 break;
             }
-        } 
+        }
         if ( s_stop ) {
             break;
         }
@@ -1466,7 +1277,7 @@ int main( int argc, char * argv[] )
     struct timeval tv_req_end;
     int slow_script_msec = 0;
     char time_buf[40];
-    
+
 #ifdef HAVE_SIGNAL_H
 #if defined(SIGPIPE) && defined(SIG_IGN)
     signal(SIGPIPE, SIG_IGN);
@@ -1477,8 +1288,12 @@ int main( int argc, char * argv[] )
     tsrm_startup(1, 1, 0, NULL);
 #endif
 
+#ifdef ZEND_SIGNALS
+	zend_signal_startup();
+#endif
+
     if (argc > 1 ) {
-        if ( parse_opt( argc, argv, &climode, 
+        if ( parse_opt( argc, argv, &climode,
                 &php_ini_path, &php_bind ) == -1 ) {
             return 1;
         }
@@ -1543,9 +1358,9 @@ int main( int argc, char * argv[] )
     }
 
     LSAPI_Init();
-   
+
     LSAPI_Init_Env_Parameters( NULL );
-    lsapi_mode = 1; 
+    lsapi_mode = 1;
 
     slow_script_msec = LSAPI_Get_Slow_Req_Msecs();
 
@@ -1559,16 +1374,16 @@ int main( int argc, char * argv[] )
         if ( slow_script_msec ) {
             gettimeofday( &tv_req_begin, NULL );
         }
-        ret = processReq(TSRMLS_C);
+        ret = processReq();
         if ( slow_script_msec ) {
             gettimeofday( &tv_req_end, NULL );
-            n = ((long) tv_req_end.tv_sec - tv_req_begin.tv_sec ) * 1000 
+            n = ((long) tv_req_end.tv_sec - tv_req_begin.tv_sec ) * 1000
                 + (tv_req_end.tv_usec - tv_req_begin.tv_usec) / 1000;
             if ( n > slow_script_msec )
             {
                 strftime( time_buf, 30, "%d/%b/%Y:%H:%M:%S", localtime( &tv_req_end.tv_sec ) );
                 fprintf( stderr, "[%s] Slow PHP script: %d ms\n  URL: %s %s\n  Query String: %s\n  Script: %s\n",
-                         time_buf, n,  LSAPI_GetRequestMethod(), 
+                         time_buf, n,  LSAPI_GetRequestMethod(),
                          LSAPI_GetScriptName(), LSAPI_GetQueryString(),
                          LSAPI_GetScriptFileName() );
 
@@ -1579,7 +1394,7 @@ int main( int argc, char * argv[] )
             break;
         }
     }
-    php_module_shutdown(TSRMLS_C);
+    php_module_shutdown();
 
 #ifdef ZTS
     tsrm_shutdown();
@@ -1655,11 +1470,7 @@ zend_module_entry litespeed_module_entry = {
 static int add_associate_array( const char * pKey, int keyLen, const char * pValue, int valLen,
                          void * arg )
 {
-#if PHP_MAJOR_VERSION < 7
-    add_assoc_string_ex((zval *)arg, (char *)pKey, keyLen+1, (char *)pValue, 1);
-#else
     add_assoc_string_ex((zval *)arg, (char *)pKey, keyLen, (char *)pValue);
-#endif        
     return 1;
 }
 
@@ -1713,16 +1524,12 @@ PHP_FUNCTION(litespeed_response_headers)
                 headerBuf[len] = 0;
                 if ( len ) {
                     while( isspace(*++p));
-#if PHP_MAJOR_VERSION < 7
-                    add_assoc_string_ex(return_value, headerBuf, len+1, p, 1);
-#else
                     add_assoc_string_ex(return_value, headerBuf, len, p);
-#endif        
                 }
             }
         }
         h = zend_llist_get_next_ex(&SG(sapi_headers).headers, &pos);
-    }  
+    }
 }
 
 /* }}} */
@@ -1732,7 +1539,7 @@ PHP_FUNCTION(litespeed_response_headers)
    Fetch all loaded module names  */
 PHP_FUNCTION(apache_get_modules)
 {
-    static const char * mod_names[] = 
+    static const char * mod_names[] =
     {
         "mod_rewrite", "mod_mime", "mod_headers", "mod_expires", "mod_auth_basic", NULL
     };
@@ -1744,10 +1551,10 @@ PHP_FUNCTION(apache_get_modules)
     array_init(return_value);
     while( *name )
     {
-        add_next_index_string(return_value, *name 
+        add_next_index_string(return_value, *name
 #if PHP_MAJOR_VERSION < 7
                                         , 1
-#endif        
+#endif
         );
         ++name;
     }
